@@ -1,16 +1,12 @@
 lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,  
-                 pentype='Ridge', L.user=NULL, f_t=NULL, Q=NULL, a=10^3,
+                 pentype='Ridge', L.user=NULL, f_t=NULL, Q=NULL, phia=10^3,
                  se=FALSE, ...)
 {
-  require(nlme)
-  require(magic)
-  lobj=TRUE
-  pd1=pd2=pd3=pd4=pd5=pd6=NULL
-  W<- as.matrix(funcs)
-  norm<- sqrt(diag(W%*%t(W)))
-  W<- W/norm
-  K<- ncol(W) 
+  pd1 = pd2 = pd3 = pd4 = pd5 = pd6 = NULL
   
+  #Determining K, converting W, Y, id and t to matrix
+  W<- as.matrix(funcs)
+  K<- ncol(W) 
   Y<- as.matrix(Y)
   id<- as.matrix(subj)
   t<-as.matrix(t)
@@ -20,10 +16,6 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
   if(dim(id)[2]>1) return(cat("Error: No. of column for subj cannot be greater than 1. \nThe lpeer() will not proceed further.\n"))
   if(dim(t)[2]>1) return(cat("Error: No. of column for t cannot be greater than 1. \nThe lpeer() will not proceed further.\n"))
   
-  Yl<- dim(Y)[1]
-  idl<- dim(id)[1]
-  tl<- dim(t)[1]
-  
   #Check 2: Do check for intercept in X matrix
   if (!is.null(covariates)){
     covariates<- as.matrix(covariates)
@@ -33,6 +25,9 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
   X<-  cbind(1, covariates)
   
   #Check 3: Check the dimension of Y, id, t, W and X
+  Yl<- dim(Y)[1]
+  idl<- dim(id)[1]
+  tl<- dim(t)[1]
   chk.eq<- ifelse(Yl==idl & idl==tl & tl==nrow(W), 0 ,1)
   if(chk.eq==1) return(cat("Error: At least one of (1) length of Y, (2) lenght of subj, (3) length of \nt, and (4) number of row of funcs are not equal.\n The lpeer() will not proceed further.\n"))
   if(!is.null(covariates) & Yl!=nrow(cbind(X,X))) return(cat("Error: length of Y and number of rows of X is not equal.\n The lpeer() will not proceed further.\n"))
@@ -69,53 +64,66 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
   X.s<- W.e + 1; X.e<- dim(X)[2]+W.e; X<- as.matrix(tdata[,c(X.s:X.e)])
   f_t.s<- X.e + 1; f_t.e<- dim(f_t)[2]+X.e; f_t<- as.matrix(tdata[,c(f_t.s:f_t.e)])
   
+  #Determining N and NT
   N<- length(unique(id))
   NT<- length(Y)
   
-  #Check 5.1: Compatibility of Q and W matrix
+  #Checking entry for pentype
+  pentypecheck<- toupper(pentype) %in% c('DECOMP', 'DECOMPOSITION', 'RIDGE', 'D2', 'USER')
+  if(!pentypecheck)  return (cat("Error: Specify valid object for argument PENTYPE.\n"))
+  
+  #Check 5: Some checking/processing for decomposition type of penalty
   if(toupper(pentype)=='DECOMP' | toupper(pentype)=='DECOMPOSITION'){
+    
+    
+    #5.1: Removing rows containing missing and infinite values
+    Q<- Q[which(apply(is.na(Q), 1, sum)==0),]
+    Q<- Q[which(apply(is.infinite(Q), 1, sum)==0),]
+    
+    #5.2: Compatibility of Q and W matrix
     if (!comm.pen) 
     {
       if(ncol(Q)!=(d+1)*ncol(W)) return(cat('Error: For different penalty, number of columns of Q need to be (d+1) \ntimes of number of columns of funcs.\nThe lpeer() will not proceed further.\n'))
     }
     if (comm.pen) 
     {
-      if(ncol(Q)!=ncol(W)) return(cat('Error: For common penalty, number of columns of pred and Q need to be equal.\nThe lpeer() will not proceed further.\n'))
+      if(ncol(Q)!=ncol(W)) return(cat('Error: For common penalty, number of columns of func and Q need to be equal.\nThe lpeer() will not proceed further.\n'))
       Q1<- Q
       for(i in 1:d) Q1<- cbind(Q1, Q)
       Q<- Q1
     }
     
-    #Check 5.2: Removing rows containing missing and infinite values
-    Q<- Q[which(apply(is.na(Q), 1, sum)==0),]
-    Q<- Q[which(apply(is.infinite(Q), 1, sum)==0),]
-    #Q<- matrix(Q, ncol=K)
-    
-    #Check 5.3: Singularity of Q matrix
+    #5.3: Singularity of Q matrix
     Q.eig<- abs(eigen(Q %*% t(Q))$values)
     if(any(Q.eig<1e-12)) return(cat('Error: Q matrix is singular or near singular.\nThe lpeer() will not proceed further.\n'))
+    
+    #5.4: Checking for phia
+    if(!exists("phia")) return (cat("Error: Specify valid object for argument PHIA.\n"))
+    if(!is.numeric(phia)|is.matrix(phia)|is.matrix(phia)) return (cat("Error: Specify valid object for argument PHIA.\n"))
   }
   
-  #Check 6.1: Dimension of L matrix
+  #Check 6: Some checking/processing for user type of penalty
   if(toupper(pentype)=='USER'){
+    L<- L.user
+    
+    #6.1: Removing rows containing missing and infinite values
+    L<- L[which(apply(is.na(L), 1, sum)==0),]
+    L<- L[which(apply(is.infinite(L), 1, sum)==0),]
+    
+    #6.2: Dimension of L matrix
     if (!comm.pen) 
     {
-      if(ncol(L.user)!=(d+1)*ncol(W)) return(cat('Error: For different penalty, number of columns of L need to be (d+1) \ntimes of number of columns of pred.\nThe lpeer() will not proceed further.\n'))
+      if(ncol(L)!=(d+1)*ncol(W)) return(cat('Error: For different penalty, number of columns of L need to be (d+1) \ntimes of number of columns of func.\nThe lpeer() will not proceed further.\n'))
     }
     if (comm.pen) 
     {
-      if(ncol(L.user)!=ncol(W)) return(cat('Error: For common penalty, number of columns of pred and L.user need to be equal.\nThe lpeer() will not proceed further.\n'))
-      L1.user<- L.user
-      for(i in 1:d) L1.user<- cbind(L1.user, L.user)
-      L.user<- L1.user
+      if(ncol(L)!=ncol(W)) return(cat('Error: For common penalty, number of columns of func and L.user need to be equal.\nThe lpeer() will not proceed further.\n'))
+      L1<- L
+      for(i in 1:d) L1<- adiag(L1, L)
+      L<- L1
     }
     
-    #Check 6.2: Removing rows containing missing and infinite values
-    L<- L[which(apply(is.na(L), 1, sum)==0),]
-    L<- L[which(apply(is.infinite(L), 1, sum)==0),]
-    L<- matrix(L, ncol=K)
-    
-    #Check 6.3: Singularity of L'L matrix
+    #6.3: Singularity of L'L matrix
     LL<- t(L)%*%L
     LL.eig<- abs(eigen(LL %*% t(LL))$values)
     if(any(LL.eig<1e-12)) return(cat("Error: L'L matrix is singular or near singular.\nThe lpeer() will not proceed further.\n"))
@@ -140,7 +148,7 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
     if(toupper(pentype)=='DECOMP' | toupper(pentype)=='DECOMPOSITION'){
       tQ<- Q[,(i*K+1):((i+1)*K)] 
       tP_Q <- t(tQ) %*% solve(tQ %*% t(tQ)) %*% tQ
-      tL_PEER<- a*(diag(K)- tP_Q) + 1*tP_Q
+      tL_PEER<- phia*(diag(K)- tP_Q) + 1*tP_Q
       rm(tQ); rm(tP_Q)
     } else
       if(toupper(pentype)=='RIDGE'){
@@ -150,13 +158,16 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
           tL_PEER<- D.2
         } else
           if(toupper(pentype)=='USER'){
-            tL_PEER<- L.user
+            tL_PEER<- L[(i*K+1):((i+1)*K),(i*K+1):((i+1)*K)]
           } 
-    v<- diag(K)
-    if(K>N) v<-  svd((data.matrix(W)*f_t[,(i+1)])%*% solve(tL_PEER))$v
-    assign(paste('W',i+1, '_PEER', sep=''), 
-           (data.matrix(W)*f_t[,(i+1)])%*% solve(tL_PEER) %*% v)
-    rm(tL_PEER) ; rm(v)
+    
+    v <- diag(K)
+    if (K > N) 
+      v <- svd((data.matrix(W) * f_t[, (i + 1)]) %*% solve(tL_PEER))$v
+    assign(paste("W", i + 1, "_PEER", sep = ""), (data.matrix(W) * 
+      f_t[, (i + 1)]) %*% solve(tL_PEER) %*% v)
+    rm(tL_PEER)
+    rm(v)
   }
   
   #Generate Z
@@ -181,16 +192,15 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
   
   #Fitting the model
   out_PEER<- lme(fixed=Y~X-1, random=list(id.bd1=tXX), ... ) 
-  
   cat('The fit is successful.\n')
   
+  #Extracting the estimates
   Gamma_PEER<-matrix(out_PEER$coeff$random$id.bd1, ncol=1)
-  
   for(i in 0:d) {
     if(toupper(pentype)=='DECOMP' | toupper(pentype)=='DECOMPOSITION'){
       tQ<- Q[,(i*K+1):((i+1)*K)] 
       tP_Q <- t(tQ) %*% solve(tQ %*% t(tQ)) %*% tQ
-      tL_PEER<- a*(diag(K)- tP_Q) + 1*tP_Q
+      tL_PEER<- phia*(diag(K)- tP_Q) + 1*tP_Q
       rm(tQ); rm(tP_Q)
     } else
       if(toupper(pentype)=='RIDGE'){
@@ -200,30 +210,34 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
           tL_PEER<- D.2
         } else
           if(toupper(pentype)=='USER'){
-            tL_PEER<- L.user
+            tL_PEER<- L[(i*K+1):((i+1)*K),(i*K+1):((i+1)*K)]
           } 
-    v<- diag(K)
-    if(K>N) v<-  svd((data.matrix(W)*f_t[,(i+1)])%*% solve(tL_PEER))$v
-    
-    
-    r<- ncol(v)
-    tGamma.PEER.hat<- matrix(Gamma_PEER[(i*r+1):((i+1)*r)],ncol=1)
-    
-    tGammaHat <- solve(tL_PEER) %*% v %*%tGamma.PEER.hat
-    if(i==0) GammaHat<- matrix(tGammaHat ,ncol=1)
-    if(i>0) GammaHat<- cbind(GammaHat, tGammaHat)
-    rm(tL_PEER); rm(v); rm(tGammaHat)
+    v <- diag(K)
+    if (K > N) 
+      v <- svd((data.matrix(W) * f_t[, (i + 1)]) %*% solve(tL_PEER))$v
+    r <- ncol(v)
+    tGamma.PEER.hat <- matrix(Gamma_PEER[(i * r + 1):((i + 
+      1) * r)], ncol = 1)
+    tGammaHat <- solve(tL_PEER) %*% v %*% tGamma.PEER.hat
+    if (i == 0) 
+      GammaHat <- matrix(tGammaHat, ncol = 1)
+    if (i > 0) 
+      GammaHat <- cbind(GammaHat, tGammaHat)
+    rm(tL_PEER)
+    rm(v)
+    rm(tGammaHat)
   }
   colnames(GammaHat)<- paste('Gamma', 0:d, sep='')
-  
   BetaHat<- summary(out_PEER)$tTable[,1]
   names(BetaHat)<- c('Intercept', colnames(covariates))
-  
   fitted.vals<- summary(out_PEER)$fitted
+  
+  #Extracting model diagnostics
   logLik<- summary(out_PEER)$logLik
   AIC<- summary(out_PEER)$AIC
   BIC<- summary(out_PEER)$BIC
   
+  #Extracting lambda and variance
   tVarCorr<- nlme:::VarCorr(out_PEER, rdig=4)[,2]
   for(i in 0:d) assign(paste('lambda', i, sep=''), 
                        1/ as.numeric(unique(tVarCorr[(i*r+1):((i+1)*r)])))
@@ -234,23 +248,32 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
     if(i>0) lambda<- c(lambda, tLambda)
   }
   names(lambda)<- paste('lambda', 0:d, sep='')
-  
   print(lambda)
-  
-  
   sd_int.est<- as.numeric(unique(tVarCorr[((d+1)*r+1):((d+1)*r+N)]))
   sigma<- out_PEER$sigma
-  
-  ###---- Standard Error
   Sigma.u<- sd_int.est
   sigma.e<- sigma
   
+  #Returning output when se=F
+  if(!se){
+    status<- 0
+    ret <- list(out_PEER, BetaHat,  fitted.vals,  
+                GammaHat, AIC, BIC, logLik, 
+                lambda, N, K, NT, Sigma.u, sigma.e, d, status)
+    names(ret)<- c("fit", "BetaHat", "fitted.vals",
+                   "GammaHat", "AIC", "BIC", "logLik",
+                   "lambda", "N", "K", "TotalObs", "Sigma.u", "sigma", "d", "status")
+    class(ret) <- "lpeer"
+    return(ret)
+  }
+  
+  ###---- Standard Error
   for(i in 0:d)
   {
     if(toupper(pentype)=='DECOMP' | toupper(pentype)=='DECOMPOSITION'){
       tQ<- Q[,(i*K+1):((i+1)*K)] 
       tP_Q <- t(tQ) %*% solve(tQ %*% t(tQ)) %*% tQ
-      tL_PEER<- a*(diag(K)- tP_Q) + 1*tP_Q
+      tL_PEER<- phia*(diag(K)- tP_Q) + 1*tP_Q
       rm(tQ); rm(tP_Q)
     } else
       if(toupper(pentype)=='RIDGE'){
@@ -260,7 +283,7 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
           tL_PEER<- D.2
         } else
           if(toupper(pentype)=='USER'){
-            tL_PEER<- L.user
+            tL_PEER<- L[(i*K+1):((i+1)*K),(i*K+1):((i+1)*K)]
           } 
     tsigma<- as.numeric(unique(tVarCorr[(i*r+1):((i+1)*r)]))
     if(i==0) LL.inv<- tsigma^2*solve(t(tL_PEER)%*%tL_PEER) 
@@ -272,18 +295,6 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
   
   rand.var<- Sigma.u^2*(Z%*%t(Z))
   
-  if(!se){
-    status<- 0
-    ret <- list(out_PEER, BetaHat,  fitted.vals,  
-                GammaHat, AIC, BIC, logLik, lobj, 
-                lambda, N, K, NT, Sigma.u, sigma.e, d, status)
-    names(ret)<- c("fit", "BetaHat", "fitted.vals",
-                   "GammaHat", "AIC", "BIC", "logLik", "lpeerobj",
-                   "lambda", "N", "K", "TotalObs", "Sigma.u", "sigma", "d", "status")
-    class(ret) <- "lpeer"
-    return(ret)
-  }
-  
   V.1<- W1%*%LL.inv%*%t(W1)+rand.var+sigma.e^2*diag(rep(1, NT))
   V<- rand.var+sigma.e^2*diag(rep(1, NT))
   
@@ -307,13 +318,15 @@ lpeer<- function(Y, subj, t, funcs, covariates=NULL, comm.pen=TRUE,
   } 
   colnames(se.Gamma)<- paste('Gamma', 0:d, sep='')
   
+  #Returning output when se=T
   status<- 1
   ret <- list(out_PEER, BetaHat,  se.Beta, Beta, fitted.vals,  
-              GammaHat, se.Gamma, AIC, BIC, logLik, V.1, lobj,
+              GammaHat, se.Gamma, AIC, BIC, logLik, V.1,
               V, lambda, N, K, NT, Sigma.u, sigma.e, d, status)
   names(ret)<- c("fit", "BetaHat", "se.Beta", "Beta", "fitted.vals",
-                 "GammaHat", "se.Gamma", "AIC", "BIC", "logLik", "V1", "lpeerobj",
+                 "GammaHat", "se.Gamma", "AIC", "BIC", "logLik", "V1",
                  "V", "lambda", "N", "K", "TotalObs", "Sigma.u", "sigma", "d", "status")
+  
   class(ret) <- "lpeer"
   ret
 }
\ No newline at end of file
