@@ -46,13 +46,23 @@ predict.pffr <- function(object,
     call <- match.call()
     nyindex <- object$pffr$nyindex
 
+    ## warn if any entries in ... are not arguments for predict.gam 
+    dots <- list(...)
+    if(length(dots)){
+        validDots <- names(formals(predict.gam))
+        notUsed <- names(dots)[!(names(dots) %in% validDots)]
+        if(length(notUsed))
+            warning("Arguments <", paste(notUsed, collapse=", "), "> supplied but not used." )
+    }
+    
     
     if(!missing(newdata)){
         nobs <- nrow(as.matrix(newdata[[1]]))
         
         # check if the supplied data already has the shape expected by predict.gam
         # and dispatch immediately if so (need this so summary works as expected!)
-        if(!(all(names(newdata) %in% names(object$model)))){
+        if(!(all(names(newdata) %in% names(object$model))) | 
+          !(paste0(object$pffr$yindname,".vec") %in% names(newdata))){
             # check lengths
             stopifnot(length(unique(sapply(newdata, function(x) ifelse(is.matrix(x), nrow(x), length(x))))) ==1)
             #        #FIXME: better leave this check to predict.gam....  
@@ -71,14 +81,18 @@ predict.pffr <- function(object,
             # which covariates occur in which terms?
             varmap <- sapply(names(object$pffr$labelmap), function(x) all.vars(formula(paste("~", x))))
             
-            for(cov in names(newdata)){
-                #find the term(s) its associated with
+            # don't include response
+            covnames <- names(newdata)[names(newdata)!=deparse(object$formula[[2]])]
+            for(cov in covnames){
+                #find the term(s) <cov> is associated with
                 trms <- which(sapply(varmap, function(x) any(grep(paste("^",cov,"$",sep=""), x)))) 
-                if(!is.null(terms)) trms <- trms[names(trms) %in% terms]
+                if(!is.null(dots$terms)) trms <- trms[names(trms) %in% dots$terms]
                 if(length(trms)!=0){
                     for(trm in trms){
                         is.ff <- trm %in% object$pffr$where$where.ff
                         is.sff <- trm %in% object$pffr$where$where.sff
+                        is.ffpc <- trm %in% object$pffr$where$where.ffpc
+                        is.pcre <- trm %in% object$pffr$where$where.pcre
                         #if ff(X) or sff(X), generate (X.mat), X.tmat, X.smat, L.X ...
                         if(is.ff){
                             ff <- object$pffr$ff[[grep(paste(cov,"[,\\)]",sep=""), names(object$pffr$ff))]]
@@ -104,7 +118,7 @@ predict.pffr <- function(object,
                                 gamdata[[paste(cov, ".tmat", sep="")]] <- 
                                         matrix(rep(ff$yind, times=nobs), ncol=length(ff$xind), nrow=nobs*nyindex)
                                 gamdata[[paste("L.", cov, sep="")]] <-  
-                                        (predL*newdata[[cov]])[rep(1:nobs, e=nyindex),]
+                                        (predL*newdata[[cov]])[rep(1:nobs, each=nyindex),]
                             }
                         }
                         if(is.sff){
@@ -127,9 +141,28 @@ predict.pffr <- function(object,
                                 gamdata[[paste("L.", cov, sep="")]] <-  predL[rep(1:nobs, e=nyindex),]
                             }
                         }
-                        if(!(is.ff || is.sff)) {
+                        if(is.pcre){
+                            pcre <- object$pffr$pcre[[grep(cov, names(object$pffr$pcre))]]
+                            gamdata[[paste(cov, ".vec", sep="")]] <- rep(pcre$id, each=nyindex)
+                            for(nm in colnames(pcre$efunctions)){
+                                gamdata[[nm]] <- pcre$efunctions[rep(1:nyindex, times=nobs), nm]
+                            }
+                        }
+                        if(is.ffpc){
+                            ffpc <- object$pffr$ffpc[[grep(paste(cov,"[,\\)]",sep=""), names(object$pffr$ffpc))]]
+                            
+                            # Xc' = Phi xi' + error --> get loadings for new data:
+                            Xct <- t(newdata[[cov]]) - as.vector(ffpc$meanX)
+                            xiMat <- t(qr.coef(qr(ffpc$PCMat), Xct))
+                            colnames(xiMat) <- paste(make.names(cov),".PC", 1:ncol(xiMat), sep="")
+                            xiMat <- xiMat[rep(1:nobs, each=nyindex), ]
+                            for(nm in colnames(xiMat)){
+                                gamdata[[nm]] <- xiMat[,nm] 
+                            }
+                        }
+                        if(!(is.ff | is.sff | is.ffpc | is.pcre)) {
                             #just repeat each entry nyindex-times to correspond to vec(<Response>)  
-                            gamdata[[cov]] <- drop(newdata[[cov]])[rep(1:nobs, e=nyindex)]
+                            gamdata[[cov]] <- drop(newdata[[cov]])[rep(1:nobs, each=nyindex)]
                         }
                         
                     } 
@@ -138,9 +171,6 @@ predict.pffr <- function(object,
             gamdata <- list2df(gamdata)
             call[["newdata"]] <- gamdata
         } 
-        
-        
-        
     } else {
         call$newdata <- eval(call$newdata)
         nobs <- object$pffr$nobs
@@ -165,13 +195,17 @@ predict.pffr <- function(object,
     if(reformat){
         
         if(missing(newdata) && !is.null(object$pffr$missingind)){
-            #add NAs at the appropriate locations so that fits are nobs x nyindex:
+            #pad with NAs at the appropriate locations so that fits are nobs x nyindex:
             insertNA <- function(x){
-                tmp <- rep(NA, nobs*object$pffr$nyindex)
-                tmp[-object$pffr$missingind] <- x
-                return(tmp)     
+                if(length(x) != nobs*object$pffr$nyindex){
+                    tmp <- rep(NA, nobs*object$pffr$nyindex)
+                    tmp[-object$pffr$missingind] <- x
+                    return(tmp)
+                } else {
+                    return(x)
+                }
             }
-        } else insertNA <- function(x) x
+        } else insertNA <- function(x) return(x)
         
         
         if(se.fit){
@@ -232,7 +266,14 @@ residuals.pffr <- function (object, reformat=TRUE, ...)
     if (!inherits(object, "pffr")) 
         stop("`object' is not of class \"pffr\"")
     ret <- mgcv:::residuals.gam(object, ...)
-    if(reformat) ret <- matrix(ret, nrow=object$pffr$nobs, ncol=object$pffr$nyindex, byrow=TRUE)
+    if(reformat){
+        if(!(length(ret)==object$pffr$nobs*object$pffr$nyindex)){
+            tmp <- rep(NA, object$pffr$nobs*object$pffr$nyindex)
+            tmp[-object$pffr$missingind] <- ret
+            ret <- tmp
+        }
+        ret <- matrix(ret, nrow=object$pffr$nobs, ncol=object$pffr$nyindex, byrow=TRUE)  
+    } 
     return(ret)
 }
 
@@ -255,7 +296,7 @@ fitted.pffr <- function (object, reformat=TRUE, ...)
         stop("`object' is not of class \"pffr\"")
     ret <- object$fitted.values
     if(reformat){
-        if(!is.null(object$pffr$missingind)){
+        if(!(length(ret)==object$pffr$nobs*object$pffr$nyindex)){
             tmp <- rep(NA, object$pffr$nobs*object$pffr$nyindex)
             tmp[-object$pffr$missingind] <- ret
             ret <- tmp
@@ -485,7 +526,7 @@ coef.pffr <- function(object, raw=FALSE, se=TRUE, freq=FALSE, sandwich=FALSE,
             res <- residuals(object)
             if(is.null(Ktt)){
                 # get estimate of Cov(eps_i(t)) = K(t,t')
-                stopifnot(require(Matrix))
+                # stopifnot(require(Matrix))
                 Ktt <- Reduce("+",  lapply(1:nrow(res), function(i) tcrossprod(res[i,])))/nrow(res)
             }
             #Chen/Wang, Sec. 2.1: M = X' V^-1 (Y-eta)(Y-eta)' V^-1 X  with V ^-1 = diag(sigma^-2) 
